(function(window, $) {
	/**
	 * 1. Carrousel类,用于创建一个Carrousel对象
	 * @class
	 * @param [string]    	opt.id			必需						Carrousel控件ID
	 * @param [int]			opt.height		必需						控件高度
	 * @param [int]			opt.width  		可选		默认100%			控件宽度
	 * @param [int]			opt.visible		可选  	默认1			显示区包含的item的个数
	 * @param [int]			opt.time		可选  	默认300ms		回弹时间
	 * @param [function]		opt.onchange	可选
	 * @param [function]		opt.autoSwitch	可选 	默认10s,0disable	自动切换

	 * @return [object]	 产生的Carrousel对象
	 * @description  1.
	 *
	 * @example
	 */
	var Carrousel = function(opt) {
		this.id = opt.id;
    this.scrollviewY = opt.scrollviewY;
		this.width = opt.width;
		this.visible = opt.visible || 1;
		this.height = opt.height;
		this.time = opt.time || 200;
		this.onchange = opt.onchange;
		this.current = 0;
		this.autoSwitch = typeof (opt.autoSwitch) == "undefined" ? 5000 : opt.autoSwitch;
		$.am.components.push(this);
		$.am.Component.call(this, opt);
	};
	Carrousel.prototype = {
		componentInit : function() {
			this.$ = $("#" + this.id).addClass("am-touchable");
			this.$inner = this.$.children(".am-carrousel-inner");
			var cssSet = {};
			if (this.width) {
				cssSet.width = this.width + "px";
			}
			cssSet.height = this.height + "px";
			this.$.css(cssSet);

			var _this = this;
			this.$.bind({
				"vtouchstart" : function(event, position) {
					if (_this.$items)
						_this.start(position);
					return false;
				},
				"vtouchmove" : function(event, position) {
					if (_this.$items)
						_this.move(position);
					return false;
				},
				"vtouchend" : function(event, position) {
					if (_this.$items)
						_this.end(position);
					return false;
				}
			});
		},
		refresh : function() {
		    var w = this._itemw = (this.width || window.innerWidth) / this.visible, h = this.height;
			this.$items = this.$inner.children();
			if (this.$items.length < 1) {
				return;
			}

			this.$inner.css({
				width : w * this.$items.length + "px"
			});
			this.$items.each(function() {
			    $(this).css({
			        'width': w + "px",
			        'height': h + "px",
			        'float': "left"
			    });
			});
			this.nPos = 0;
			this.current = 0;
			this.show(0);
		},
		start : function(e) {
      if(this.scrollviewY){
        this.scrollviewY.start = [e.x, e.y];
        this.scrollviewY._start();
      }

			this.carrouselTimer && clearTimeout(this.carrouselTimer);
			this.startPos = e.x;
			this.startTime = new Date().getTime();
		},
		move: function (e) {
      //$.am.debug.log(e.y);
      if(this.scrollviewY){
        var min = Math.abs( (e.y - this.scrollviewY.start[1]) );
        $.am.debug.log("y:"+ min );
        if (min > 25){
          this.scrollviewY._move([e.x - this.scrollviewY.start[0], e.y - this.scrollviewY.start[1]]);
        }
      }

			if (e.x - this.startPos > this.current * this._itemw)
				return;
			if (e.x - this.startPos < -(this.$items.length - this.current - this.visible) * this._itemw)
				return;
			this.endPos = e.x;
			var m = this.endPos - this.startPos;
			this.nPos = -this._itemw * this.current + m;
			this.$inner.setTransformPos(this.nPos, "x");

		},
		end : function() {
      this.scrollviewY && this.scrollviewY._end();

			var m = this.endPos - this.startPos;
			var duration = new Date().getTime() - this.startTime;
			var speed = m / duration;
			// console.log(speed);
			if (m && ((m > this._itemw / 2) || speed > 0.6)) {
				var i = Math.ceil(m / this._itemw);
				this.show(this.current - i);
        this.scrollviewY && this.scrollviewY.scrollTo("top");
			} else if (m && (m < -this._itemw / 2 || speed < -0.6)) {
				var i = Math.ceil(Math.abs(m) / this._itemw);
				this.show(this.current + i);
        this.scrollviewY && this.scrollviewY.scrollTo("top");
			} else if (m) {
				this.show(this.current);
			}
			this.endPos = 'undefined';
		},
		show : function(i) {
			if (this.$items.length < 1) {
				return;
			}
			this.current = i;
			if (i < 0) {
				this.current = 0;
			}
			if (i >= this.$items.length - this.visible) {
				this.current = this.$items.length - this.visible;
			}
			var _this = this;
			this.$inner.animateTranslate3d(this.nPos, -this._itemw * this.current, "x", this.time, function() {
				_this.nPos = -_this._itemw * _this.current;
				_this.onchange && _this.onchange(_this.current);
			});

			this.carrouselTimer && clearTimeout(this.carrouselTimer);
			if (this.autoSwitch != 0) {
				this.carrouselTimer = setTimeout(function() {
					// console.log("auto move:"+ (_this.current+1));
					_this.show(i >= _this.$items.length - _this.visible ? 0 : _this.current + 1);
				}, this.autoSwitch);
			}
		}
	};
	$.am.Carrousel = Carrousel;

})(window, jQuery);



(function (window, $) {
    /**
	 * 1. Carrousel类,用于创建一个Carrousel对象
	 * @class
	 * @param [string]    	opt.id			必需						Carrousel控件ID
	 * @param [int]			opt.height		必需						控件高度
	 * @param [int]			opt.width  		可选		默认100%			控件宽度
	 * @param [int]			opt.visible		可选  	默认1			显示区包含的item的个数
	 * @param [int]			opt.time		可选  	默认300ms		回弹时间
	 * @param [function]		opt.onchange	可选
	 * @param [function]		opt.autoSwitch	可选 	默认10s,0disable	自动切换

	 * @return [object]	 产生的Carrousel对象
	 * @description  1.
	 *
	 * @example
	 */
    var Carrousel = function (opt) {
        this.id = opt.id;
        this.$ = opt.$;
        this.width = opt.width;
        this.visible = opt.visible || 1;
        this.height = opt.height;
        this.time = opt.time || 200;
        this.onchange = opt.onchange;
        this.current = 0;
        this.autoSwitch = typeof (opt.autoSwitch) == "undefined" ? 5000 : opt.autoSwitch;
        $.am.components.push(this);
        $.am.Component.call(this, opt);
    }
    Carrousel.prototype = {
        componentInit: function () {
            this.$ = this.$ || $("#" + this.id);
            this.$.addClass("am-touchable")
            this.$inner = this.$.children(".am-carrousel-inner");
            var cssSet = {};
            if (this.width) {
                this.width = cssSet.width = this.width + "px";
            }
            cssSet.height = this.height + "px";
            this.$.css(cssSet);

            var _this = this;
            var sPos = null;
            var mPos = null;
            var isMove = false;
            this.$.bind({
                "vtouchstart": function (event, position) {
                    sPos = {
                        x: position.x,
                        y: position.y
                    }
                    if (_this.$items) {
                        _this.start(sPos.x);
                    }
                    isMove = false;
                    mPos = null;
                    //return false;
                },
                "vtouchmove": function (event, position) {
                    if (!mPos) {
                        mPos = {
                            x: position.x,
                            y: position.y
                        }
                        var t = Math.abs(mPos.x - sPos.x) > Math.abs(mPos.y - sPos.y);
                        if (t) {
                            isMove = true;
                            if (_this.$items) {
                                _this.move(mPos.x);
                            }
                            return false;
                        } else {

                        }
                    } else {
                        mPos = {
                            x: position.x,
                            y: position.y
                        }
                        if (isMove) {
                            if (_this.$items) {
                                _this.move(mPos.x);
                            }
                            return false;
                        } else {

                        }
                    }
                    //position.e.preventDefault();
                    //return false;
                },
                "vtouchend": function (event, position) {
                    if (isMove && _this.$items) {
                        _this.end();//position.x
                    }
                    isMove = false;
                    sPos = null;
                    mPos = null;
                }
            });
        },
        refresh: function () {
            var innerWidth = this.$.innerWidth();
            var w = this._itemw = innerWidth / this.visible, h = this.height;
            this.$items = this.$inner.children();
            if (this.$items.length <= 1) {
                return;
            }
            this.$items.each(function () {
                $(this).css({
                    'width': w + "px",
                    'height': h + "px",
                    'float': "left"
                });
            });
            this.nPos = -w;
            this.current = 0;
            if (!this.$inner.find(".cloneItem").length) {
                this.cloned = true;
                this.$inner.prepend(this.$items.last().clone().addClass("cloneItem"));
                this.$inner.append(this.$items.first().clone().addClass("cloneItem"));
            } else {
                this.$items = this.$inner.children(":not(.cloneItem)");
            }
            this.$inner.css({
                width: w * (this.$items.length + 2) + "px"
            });
            this.show(0);
        },
        start: function (e) {
            if (this.$items.length <= 1) {
                return;
            }
            this.startPos = e;
            this.startTime = new Date().getTime();
        },
        move: function (e) {
            if (this.$items.length <= 1) {
                return;
            }
            this.carrouselTimer && clearTimeout(this.carrouselTimer);
            /*if (e - this.startPos > this.current * this._itemw)
				return;
			if (e - this.startPos < -(this.$items.length - this.current - this.visible) * this._itemw)
				return;*/
            this.endPos = e;
            var m = this.endPos - this.startPos;
            this.nPos = -this._itemw * (this.current + 1) + m;
            this.$inner.setTransformPos(this.nPos, "x");
        },
        end: function () {
            if (this.$items.length <= 1) {
                return;
            }
            var m = this.endPos - this.startPos;
            var duration = new Date().getTime() - this.startTime;
            var speed = m / duration;
            // console.log(speed);
            if (m && ((m > this._itemw / 2) || speed > 0.6)) {
                var i = Math.ceil(m / this._itemw);
                this.show(this.current - i);
            } else if (m && (m < -this._itemw / 2 || speed < -0.6)) {
                var i = Math.ceil(Math.abs(m) / this._itemw);
                this.show(this.current + i);
            } else if (m) {
                this.show(this.current);
            }
            this.endPos = 'undefined';
        },
        show: function (i) {
            if (this.$items.length < 1) {
                return;
            }
            this.current = i;
            //$("div.page-header").text("current:"+ i);
            //$("div.page-header").append(this.nPos+" | "+(-this._itemw *(this.current+1)));
            var _this = this;
            this.$inner.animateTranslate3d(this.nPos, -this._itemw * (this.current + 1), "x", this.time, function () {
                if (i < 0) {
                    _this.current = _this.$items.length - _this.visible;//0
                }
                if (i > _this.$items.length - _this.visible) {
                    _this.current = 0;//this.$items.length - this.visible;
                }
                _this.nPos = -_this._itemw * (_this.current + 1);
                _this.$inner.setTransformPos(_this.nPos, "x");
                _this.onchange && _this.onchange(_this.current);
            });

            this.carrouselTimer && clearTimeout(this.carrouselTimer);
            if (this.autoSwitch != 0) {
                this.carrouselTimer = setTimeout(function () {
                    _this.show(_this.current + 1);
                }, this.autoSwitch);
            }
        }
    }
    $.am.CarrouselLoop = Carrousel

})(window, jQuery);